package internal

import (
	"context"
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/stretchr/testify/assert"
	"net/http"
	"testing"
)

var zltx = &ZLTXV2Service{}

func config() []byte {
	c := &Config{
		AppId:     "23329",
		AppKey:    "8db16e8cc8363ed4eb4c14f9520bcc32",
		BaseUri:   "http://211.137.105.198:17100",
		NotifyUrl: "https://gateway.dev.cdlsxd.cn/yxh5api/v1/order/direct/notify",
	}
	marshal, _ := json.Marshal(c)
	return marshal
}

func TestConfig(t *testing.T) {
	t.Run("TestConfig", func(t *testing.T) {
		c := config()
		fmt.Printf("%s\n", string(c))
		assert.NotEmpty(t, c)
	})
}

func TestOrder(t *testing.T) {
	request := &proto.OrderRequest{
		Config: config(),
		Order: &proto.OrderRequest_Order{
			OrderNo:  "test_zltx_v2_7",
			Account:  "18666173766",
			Quantity: 1,
			Extra:    nil,
		},
		Product: &proto.OrderRequest_Product{
			ProductNo: "106",
			Extra:     []byte(`{}`),
		},
	}

	t.Run("TestOrder", func(t *testing.T) {
		got, err := zltx.Order(context.Background(), request)
		if err != nil {
			t.Errorf("Order() error = %v", err)
			return
		}
		fmt.Printf("%+v\n", got)
		assert.Equal(t, int(proto.Status_ING), int(got.Result.Status))
	})
}

func TestQuery(t *testing.T) {
	request := &proto.QueryRequest{
		Config: config(),
		Order: &proto.QueryRequest_Order{
			OrderNo: "test_zltx_v2_7",
			TradeNo: "", // 731095414466813953
			Account: "",
			Extra:   nil,
		},
	}
	t.Run("TestQuery", func(t *testing.T) {
		got, err := zltx.Query(context.Background(), request)
		if err != nil {
			t.Errorf("Query() error = %v", err)
			return
		}
		fmt.Printf("%+v\n", got)
		assert.Equal(t, int(proto.Status_SUCCESS), int(got.Result.Status))
	})
}

func TestNotify(t *testing.T) {
	bodyBytes := []byte(`{"tradeStatus":"SUCCESS","orderNo":"731095414466813953","tradeStateDesc":"成功","mchId":23329,"outTradeNo":"test_zltx_v2_7","rechargeAccount":"18666173766","unitPrice":0.1,"attach":""}`)
	headerBytes := []byte(`{"Accept-Encoding":["gzip"],"Authorization":["39859e5d71a1e7a9f69a6d00cfe79a7a"],"Connection":["close"],"Content-Length":["186"],"Content-Type":["application/json"],"User-Agent":["GuzzleHttp/6.5.5 curl/7.69.1 PHP/7.2.34"],"X-Envoy-Attempt-Count":["1"],"X-Envoy-External-Address":["120.79.35.82"],"X-Forwarded-Client-Cert":["By=spiffe://cluster.local/ns/gopro/sa/default;Hash=7e17458f89c270f74ca6039bfd8c091de2924d2f95f3e415f9c6f95e67932798;Subject=\"\";URI=spiffe://cluster.local/ns/istio-system/sa/istio-ingressgateway-go"],"X-Forwarded-For":["120.79.35.82, 112.124.3.46"],"X-Forwarded-Proto":["https"],"X-Real-Ip":["112.124.3.46"],"X-Remoteaddr":["172.21.0.1"],"X-Request-Id":["69a03321-e633-466b-808e-7e822232cf12"]}`)

	in := &proto.NotifyRequest{
		Config:  config(),
		Queries: nil,
		Headers: headerBytes,
		Body:    bodyBytes,
	}

	t.Run("TestNotify", func(t *testing.T) {
		got, err := zltx.Notify(context.Background(), in)
		if !assert.Nil(t, err) {
			t.Errorf("Notify() error = %v", err)
			return
		}

		responseHeaders := make(http.Header)
		_ = json.Unmarshal([]byte(got.Headers), &responseHeaders)
		for s, strings := range responseHeaders {
			t.Logf("%s: %v", s, strings)
		}

		t.Logf("%s \n", got.String())
		assert.Equal(t, int(proto.Status_SUCCESS), int(got.Result.Status))
	})
}
